#include <stdio.h>
#include <mex.h>
#include "nc.h"

#include "ncmat.h"

long
ncmat_get_size( NCA* nca, long* mrows, long* ncols ) 
{
    DIM* dim;
    DIM* dimLast;
    
    if( nca && nca->dim ) {
        for( dim = nca->dim; dim; dim = dim->next ) {
            dimLast = dim;
        }
        *mrows = dimLast->size;
        *ncols = nca->nels / *mrows;
        return 0;
    }
    else if( nca->nels == 1 ) {
        *mrows = *ncols = 1;
        return 0;
    }
    else {
        *mrows = *ncols = -1;
        return -1;
    }
}

Matrix*
ncmat_get_matrix( const char* fan, const char* units ) 
{
    long rc, i;
    long mrows, ncols, nelems;
    double* zin = 0;
    Matrix* mat = 0;
    NCA* nca = 0;
    double* pr = 0;
    
#ifdef debug
    fprintf( stderr, "dbg:  in ncmat_get_matrix( \"%s\", \"%s\" )\n",
             fan, units ? units : "" );
    fflush( stderr );
#endif

    nca = nasParse( 1, (char**)&fan );
    if( !nca ) {
        mat = 0;
        FILELINE;
        fprintf(stderr, "error:  error paring fan specification \"%s\"\n", fan );
        fflush(stderr);
    }
    else {
        rc = ncmat_get_size( nca, &mrows, &ncols );
        if( rc < 0 || mrows*ncols <= 0 ) {
            mat = 0;
            FILELINE;
            fprintf(stderr, "error:  error finding size \"%s\","
                    "rc = %d, mrows = %ld, ncols = %ld\n", fan, rc, mrows, ncols );
            fflush(stderr);
        }
        else {
#ifdef debug
            fprintf(stderr, "dbg:  mrows = %ld, ncols = %ld\n", mrows, ncols );
            fflush( stderr );
#endif
            zin = get_var( nca, (char*)units );
            if( nca->type == NC_CHAR ) {
                /* interchange rows and columns so typical string is a row vector */
                long tmp = ncols;
                ncols = mrows;
                mrows = tmp;
                mat = mxCreateFull( mrows, ncols, 0 );
                mxSetString( mat );
            }
            else {
                mat = mxCreateFull( mrows, ncols, 0 );
            }
            pr = mxGetPr( mat );
            if( !zin || !mat || !pr ) {
                mat = 0;
                FILELINE;
                fprintf(stderr, "error:  error getting data for \"%s\"\n", fan );
                fflush(stderr);
            }
            else {
                nelems = mrows*ncols;
                for( i = 0; i < nelems; i++ ) {
                    pr[i] = zin[i];
                }
                free( zin );
            }
        }
    }
    return mat;
}

long
ncmat_put_matrix( const char* fan, const char* units, Matrix* mat ) 
{
    NCA* nca = 0;
    long mrows, ncols, nels;
    long rc, i;
    double* pr = 0;
    nc_type att_type;
    
#ifdef debug
    fprintf( stderr, "dbg:  in ncmat_put_matrix( \"%s\", \"%s\", \"%s\" )\n",
             fan, units ? units : "", "Matrix*" );
    fflush( stderr );
#endif
    mrows = mxGetM( mat );
    ncols = mxGetN( mat );
    pr = mxGetPr( mat );
    nels = mxGetM(mat) * mxGetN(mat);
    
    nca = nasParse( 1, (char**)&fan );

    /* make sure everything is okay */
    if( !nca ) {
        FILELINE;
        fprintf( stderr, "error:  invalid nca returns using \"%s\"\n", fan );
        fflush( stderr );
        rc = -1;
    }
    else if( !pr ) {
        FILELINE;
        fprintf( stderr, "error:  Matrix* does not have valid data\n" );
        fflush( stderr );
        rc = -1;
    }
    else {
        if( nca->attname ) {
            if( nca->next ) {
                fprintf( stderr, "warn:  ignoring the remaining nca's\n" );
                fflush(stderr);
            }
            if( mxIsString(mat) ) {
                att_type = NC_CHAR;
            }
            else {
                att_type = NC_DOUBLE;
            }
            nca->type = att_type;
            nca->nels = nels;
        }
        else if( nca->nels < nels ) {
            fprintf(stderr, "warn:  excess input data ignored\n" );
        }
            
        rc = put_var( pr, nca, 0, 0, (char*)units );
    }
    return rc;
}

#ifdef use_mymx
double*
mymxGetPr( const myMatrix* pm ) 
{
    if( pm ) {
        return pm->pr;
    }
    else {
        return 0;
    }
}

long
mymxGetM( const myMatrix* pm ) 
{
    if( pm ) {
        return pm->m;
    }
    else {
        return 0;
    }
}

long
mymxGetN( const myMatrix* pm ) 
{
    if( pm ) {
        return pm->n;
    }
    else {
        return 0;
    }
}

myMatrix*
mymxCreateFull( long m, long n, int cmplx_flg ) 
{
    if( m*n <= 0 ) {
        return 0;
    }
    else {
        myMatrix* pm = (myMatrix*)malloc(sizeof( myMatrix ));
        if( pm ) {
            pm->m = m;
            pm->n = n;
            pm->pr = (double*)malloc(m*n*sizeof(double));
            pm->isstr = 0;
        }
        return pm;
    }
}

int
mymxIsString( const myMatrix* pm ) 
{
    if( pm ) {
        return pm->isstr;
    }
    else {
        return 0;
    }
}

void
mymxSetString( myMatrix* pm ) 
{
    if( pm ) {
        pm->isstr = 1;
    }
}
#endif

char*
ncmat_mat2str( const Matrix* pm ) 
{
    double* pr = mxGetPr( pm );
    long len = mxGetM( pm )*mxGetN( pm );
    char* str = ncmat_d2str( pr, len );
#if 0
    fprintf(stderr, "%s at %d:  str = %s\n", __FILE__, __LINE__, str );
    fflush( stderr );
#endif
    return str;
}

char*
ncmat_d2str( const double* pr, long len )
{
    int i;
    char* str = (char*) malloc( (len + 1)*sizeof(char) );
    
    if( str ) {
#if 0        
        fprintf(stderr, "d2str reached!!!\n");
        fflush(stderr);
#endif
        for (i = 0; i < len; i++) {
            str[i] = (char)pr[i];
#if 0
            fprintf( stderr, "str[%d] = %c, pr[%d] = %f\n", i, str[i], i, pr[i]);
            fflush(stderr);
#endif
        }
        str[len] = '\0';
#if 0
        fprintf(stderr, "str = %s\n", str );
        fflush(stderr);
#endif
    }
    else {
        fprintf(stderr, "%s at %d:  cannot allocate memory for string\n", __FILE__, __LINE__);
        fflush(stderr);
    }
    return (str);
}
