#include <mex.h>
#include "ncmat.h"

#if defined(nc2mat)
void
usage() 
{
    fprintf(stderr, "usage:  m = nc2mat( fanspec [,units] )\n"
            "\n"
            "where\n"
            "\tfanspec is a fan input specification\n"
            "\tunits is a compatible udunit specification, and is optional\n"
            "and\n"
            "m is the matlab matrix representation of fanspec\n"
        );
    fflush(stderr);
}

void
mexFunction(int nlhs, Matrix* plhs[], int nrhs, Matrix* prhs[] ) 
{
    char* units = 0;
    char* fan = 0;
#ifdef debug
    fprintf(stderr, "dbg:  nlhs = %d, nrhs = %d\n", nlhs, nrhs );
    fflush(stderr);
#endif
    if( nlhs > 1 ) {
        fprintf( stderr, "can only have one output argument\n" );
        usage();
        return;
    }
    if( nrhs < 1 ) {
        fprintf( stderr, "need at least on input argument\n" );
        usage();
        return;
    }
    else if( nrhs > 2 ) {
        fprintf( stderr, "too many output arguments\n" );
        usage();
        return;
    }
    
    if( nrhs == 2 ) {
        units = ncmat_mat2str( prhs[1] );
    }
    fan = ncmat_mat2str( prhs[0] );
#ifdef debug
    fprintf( stderr, "dbg:  nc2mat(\"%s\", \"%s\")\n", fan, units ? units : "");
    fflush( stderr );
#endif
    ncopts = NC_VERBOSE;
    plhs[0] = ncmat_get_matrix( fan, units );
    return;
}

#elif defined(mat2nc)

void
usage() 
{
    fprintf(stderr, "usage:  rc = mat2nc( fanspec, dat [,units] )\n"
            "\n"
            "where\n"
            "\tfanspec is a fan input specification\n"
            "\tdat is a matrix whose size matches that of the processed fanspec\n"
            "\tunits is a compatible udunit specification, and is optional\n"
            "and\n"
            "\trc is a return code status indicator (if < 0 an error occured)\n\n"
        );
    fflush(stderr);
}

void
mexFunction(int nlhs, Matrix* plhs[], int nrhs, Matrix* prhs[] ) 
{
    char* units = 0;
    char* fan = 0;
    double* pr = 0;
#ifdef debug
    fprintf(stderr, "dbg:  nlhs = %d, nrhs = %d\n", nlhs, nrhs );
    fflush(stderr);
#endif
    if( nlhs > 1 ) {
        fprintf( stderr, "can only have one output argument\n" );
        usage();
        return;
    }
    if( nrhs < 1 ) {
        fprintf( stderr, "need at least on input argument\n" );
        usage();
        return;
    }
    else if( nrhs > 3 ) {
        fprintf( stderr, "too many output arguments\n" );
        usage();
        return;
    }
    
    if( nrhs == 3 ) {
        units = ncmat_mat2str( prhs[2] );
    }
    fan = ncmat_mat2str( prhs[0] );
#ifdef debug
    fprintf( stderr, "dbg:  mat2nc(\"%s\", [dat], \"%s\")\n", fan, units ? units : "");
    fflush( stderr );
#endif
    ncopts = NC_VERBOSE;
    plhs[0] = mxCreateFull(1, 1, 0); /* just return a scalar error code */
    pr = mxGetPr( plhs[0] );
    *pr = ncmat_put_matrix( fan, units, prhs[1] );
    return;
}

#elif defined(use_mymx)

myMatrix*
mymxCreateStringMatrix( const char* str ) 
{
    myMatrix* pm = 0;
    long i, nels;
    
    pm = mymxCreateFull( 1, strlen(str) + 1, 0);
    nels = mymxGetM( pm )*mymxGetN( pm );
    
    for(i = 0; i < nels; i++ ) {
        pm->pr[i] = str[i];
    }
    mymxSetString( pm );
    
    return pm;
}

void
test_putScalar()
{
    long rc;
    double z0mean = 8.32;
    const char* naso = "t.nc z0mean";
    const char* units = 0;
    Matrix* pm = mxCreateFull(1, 1, 0);
    double* pr = mxGetPr( pm );
    *pr = z0mean;
    
    rc = ncmat_put_matrix( naso, units, pm );
    fprintf(stdout, "ncmat_put_matrix(\"%s\", 0, %f) returned %d\n",
            naso, z0mean, rc );
}


void
test_putAttribute() 
{
    long rc;
    const char* naso = "t.nc z0mean:aTestAttribute";
    const char* dat = "hello, world";
    const char* units = 0;
    Matrix* pm = mymxCreateStringMatrix( dat );
    rc = ncmat_put_matrix( naso, units, pm );
    fprintf(stdout, "ncmat_put_matrix(\"%s\", 0, %s) returned %d\n",
            naso, dat, rc );
}

int
main( int argc, char* argv[] ) 
{
    char* nasi = 0;
    long rank = -1;
    long mrows, ncols, nelems;
    Matrix* pm = 0;
    double* pr = 0;
    long i, j, k;
    long rc;
    
    ncopts = NC_VERBOSE;

    if( argc >1 ) {
        nasi = argv[1];
        fprintf( stdout, "nasi = %s\n", nasi );

        pm = ncmat_get_matrix( nasi, 0 );
        if( pm ) {
            pr = mxGetPr( pm );
            mrows = mxGetM( pm );
            ncols = mxGetN( pm );
            nelems = mrows*ncols;
            
            if( !mxIsString( pm ) ) {
                k = 0;
                for( i = 0; i < mrows; i++ ){
                    for( j = 0; j < ncols; j++ ) {
                        fprintf( stdout, "%f ", pr[k] );
                        k++;
                    }
                    fprintf( stdout, "\n" );
                }
            }
            else {
                char* str = ncmat_mat2str( pm );
                fprintf( stdout, "%s\n", str );
                fflush( stdout );
                free( str );
            }
        }
        else {
            FILELINE;
            fprintf( stderr, "error:  invalid matrix pointer returned.\n" );
            fflush( stderr );
        }
    }
    test_putScalar();
    test_putAttribute();
    
    return 0;
}

#endif


